/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */

/**
 * @author Alexis Dorn
 * @class Arapi.app.Application
 * @extends Ext.app.Application
 * 
 * This class provides inheritance to controllers of MVC Application Architecture of Ext JS 4
 * 
 * In complex situations such as multi client software development you might extend existing controllers of your application. 
 * Doing this by using the controllers property of Ext.app.Application you always have to write own getters
 * 
 * such as
 * 
 * 		//Ext.app.Application
 * 
 * 		//Core code	
 * 		getMainController: function (){
 * 			return this.getController("core.Main");
 * 		}
 * 		
 * 		//Client code	
 * 		getMainController: function (){
 * 			return this.getController("client.Main");
 * 		}
 * 
 * 
 * Furthermore the controller property of Ext.app.Application is not inheritable.
 * 
 * 		//Ext.app.Application
 * 
 * 		//Core Code
 * 		controllers: [
 * 			"core.Main",
 * 			"core.User",
 * 			"core.Product",
 * 			"core.Category"
 * 		]
 * 
 * 		//Client Code
 * 		controllers: [
 * 			"client.Main",
 * 			"core.User",
 * 			"core.Product",
 * 			"client.Category"
 * 		]
 * 
 * 
 * Arapi.app.Application enables inheritance of controllers and neutral referencing
 * 
 * 		//Arapi.app.Application
 *  
 * 		//Core Code
 * 		arapiControllers: [
 * 			{
 * 				ref: "Main",
 * 				className: "core.Main"
 * 			},
 * 			{
 * 				ref: "User",
 * 				className: "core.User"
 * 			},
 * 			{
 * 				ref: "Product",
 * 				className: "core.Product"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "core.Category"
 * 			}
 * 		]
 * 
 * 		//Client Code
 * 		arapiControllers: [
 * 			{
 * 				ref: "Main",
 * 				className: "client.Main"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "client.Category"
 * 			}
 * 		]	
 * 		
 * 
 * In both core and client you can access to controllers by using getArapiController
 * 
 *  		//Arapi.app.Application
 * 
 * 			this.getArapiController("Main");
 * 			this.getArapiController("User");
 * 			this.getArapiController("Product");
 * 			this.getArapiController("Category");
 * 
 * 
 * 
 * Start your application with an extension of Arapi.app.Application
 * 
 * 		//Arapi.app.Application
 * 
 * 		Ext.define("MyProject.app.Application",
 * 			extend: "Arapi.app.Application",
 * 			arapiControllers: [
 * 				{
 * 					ref: "Users",
 * 					className: "core.Users"
 * 				},
 * 				(...)
 * 			] 
 * 		)
 * 
 * In app.js you can instantiate your application by
 * 	
 * 		//Arapi.app.Application
 * 
 * 		Ext.application ("MyProject.app.Application", {});		
 * 
 * 
 * This ensures that Sencha SDK builder and creator will work but you have to override the Ext.application method:
 * 
 * 		//Arapi.app.Application
 * 			
 * 		Ext.application = function(config, constFn) {
 * 			var constFn = Ext.isString(constFn) ? constFn : "Ext.app.Application";
 * 			Ext.require(constFn);
 * 			Ext.onReady(function() {
 * 				Ext.create(constFn, config);
 * 			});
 * 		};
 * 
 * 
 * Using Arapi.app.Application you must use arapiControllers instead of controllers property.
 * 
 * 
 * For examples, docs, download and unit tests have a look at <a href="http://www.alexisdorn.de/arapi-ext">http://www.alexisdorn.de/arapi-ext</a>.
 * 
 * Author: Alexis Dorn
 * 
 */

Ext.define('Arapi.app.Application', {
	extend: 'Ext.app.Application',
	onClassExtended: function(cls, data) {
		var extendFn = function (cls, data, last){
			var arapiControllers = data.arapiControllers || [],
				controllers = data.controllers || [],
				superCls = cls.superclass,
				superArapiControllers = superCls.arapiControllers || [],
				superControllers = superCls.controllers || [],
				arapiControllersMixedCollection, 
				i = 0,
				ln,
				last = last || false
			;
			
			//Merge arapiControllers with parent arapiControllers
			arapiControllers = superArapiControllers.concat(arapiControllers);
			
			arapiControllersMixedCollection = new Ext.util.MixedCollection(false, function(item){
				return item.ref;
			});
			
			for (i = 0, ln = arapiControllers.length; i < ln; ++i) {
				arapiControllersMixedCollection.add(arapiControllers[i]);
            }
			
			data.arapiControllers = arapiControllersMixedCollection.getRange();
			
			data.controllers = [];
			data.arapiControllersMixedCollection = arapiControllersMixedCollection;
			for (i = 0, ln = data.arapiControllers.length; i < ln; ++i) {
				data.controllers.push(data.arapiControllers[i]["className"]);
	        }
		};
		
		//Setting onClassExtended
		data.onClassExtended = function (){
			extendFn.apply(this || window, arguments);
		};
		
		extendFn.apply(this || window, [cls, data, true]);
	},
	/**
	 * @cfg {Boolean} debug If debug set to true, application will generate a public namespace for 
	 * instance of application <b>"debug" + this.name</b>
	 * to access application and controllers from outside (e.g. unit tests, performance tests...)
	 */
	debug: false,
	/**
	 * Gets the controller by ref defined in arapiControllers property
	 * @param {String} ref
	 * @return {Arapi.app.Controller/false}
	 */
	getArapiController: function (ref){
		var controller = this.arapiControllersMixedCollection.getByKey(ref);
		if (!Ext.isEmpty(controller)){
			return this.getController(controller.className);
		}
		return false;
	},
	constructor: function (cfg){
		this.callParent([cfg]);
		if (this.debug){
			window["debug" + this.name] = this;
		}
	},
	/**
	 * Returns if application runs in debug mode
	 * @returns {Boolean}
	 */
	isDebug: function (){
		return Ext.isBoolean(this.debug) ? this.debug : false; 
	},
	/**
     * @private
     */
	getModel: function (){
		return this.callParent(arguments);
	},
	/**
     * @private
     */
	getStore: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getView: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getController: function (){
		return this.callParent(arguments);
	}
	 /**
     * Inheritable controllers of application.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Application controllers definition</li>
     * </ul>
     * @property arapiControllers
     * @type Objects[]
     */
	/**
	 * @hide
     * @cfg {String[]} models  
     */
	/**
	 * @hide
     * @cfg {String[]} stores
     */
	/**
	 * @hide
     * @cfg {String[]} views
     */
	/**
	 * @hide
     * @cfg {String[]} controllers
     */
	/**
	 * @hide
     * @cfg {String[]} refs
     */
});