/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */

/**
 * @author Alexis Dorn
 * @class Arapi.app.Controller
 * @extends Ext.app.Controller
 * 
 * 
 * This class provides inheritance to models, stores, views and refs of MVC Application Architecture of Ext JS 4
 * 
 * In complex situations such as multi client software development you might extend existing models, stores, views and refs of your application. 
 * 
 * Furthermore you need a neutral getter without using the namespaces of core and client. 
 * 
 * In Ext.app.Controller you have to declare your models, stores and views by using an array with string elements. 
 * You cannot extend this properties in extended controllers (client controllers) of your application.
 * 
 * 		//Ext.app.Controller
 * 
 * 		//Core code
 * 		stores: [
 * 			"core.Users",
 * 			"core.Persons",
 * 			"core.Products",
 * 			"core.Category",
 * 		]
 * 
 * 		//Client code
 * 		stores: [
 * 			"client.Users",
 * 			"core.Persons",
 * 			"core.Products",
 * 			"client.Category",
 * 		]
 * 
 * In Ext.app.Controller you can access your stores with
 * 
 * 		//Ext.app.Controller
 * 
 * 		//Core code
 * 		this.getStore("core.Users"); // or this.getCoreUsersStore();
 * 		this.getStore("core.Persons"); // or this.getCorePersonsStore();
 * 		this.getStore("core.Products"); // or this.getCoreProductsStore();
 * 		this.getStore("core.Category"); // or this.getCoreCategoryStore();			
 * 		
 * 		//Client code
 * 		this.getStore("client.Users"); // or this.getClientUsersStore();
 * 		this.getStore("core.Persons"); // or this.getCorePersonsStore();
 * 		this.getStore("core.Products"); // or this.getCoreProductsStore();
 * 		this.getStore("client.Category"); // or this.getClientCategoryStore();
 *
 * So in Ext.app.Controller you need to write some neutral getters to override them in inheritance
 * 		
 *		//Ext.app.Controller
 *
 * 		//Core code
 * 		getUserStore: function (){
 * 			return this.getStore("core.Users");
 * 		}
 * 		(...)
 * 
 * 		//Client code
 * 		getUserStore: function (){
 * 			return this.getStore("client.Users");
 * 		}
 * 		(...)	
 * 
 * Arapi.app.Controller provides inheritance of stores, models, views and refs by using the properties 
 * <ul>
 * 	<li>arapiStores</li>
 * 	<li>arapiModels</li>
 * 	<li>arapiViews</li>
 * 	<li>arapiRefs</li>
 * </ul>
 * 
 * For example:
 *
 * 		//Arapi.app.Controller
 *
 * 		//Core code
 * 		arapiStores: [
 * 			{
 * 				ref: "Users",
 * 				className: "core.Users"
 * 			},
 * 			{
 * 				ref: "Persons",
 * 				className: "core.Persons"
 * 			},
 * 			{
 * 				ref: "Products",
 * 				className: "core.Products"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "core.Category"
 * 			}
 * 		]
 * 
 * 
 * 		//Client code
 * 		arapiStores: [
 * 			{
 * 				ref: "Users",
 * 				className: "client.Users"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "client.Category"
 * 			}
 * 		] 
 * 
 * In both core and client you can access to stores by using getArapiStore
 *
 * 		//Arapi.app.Controller
 *
 * 			this.getArapiStore("Users");
 * 			this.getArapiStore("Persons");
 * 			this.getArapiStore("Products");
 * 			this.getArapiStore("Category");
 * 
 * 
 * Inheritable Models and views will be defined by 
 * 		
 *		//Arapi.app.Controller
 *
 * 		arapiModels: [
 * 			{
 * 				ref: "User",
 * 				className: "core.User"
 * 			}
 * 		],
 * 		arapiViews: [
 * 			{
 * 				ref: "UserWindow",
 * 				className: "core.user.Window"
 * 			}
 * 		]
 * 
 * and you can access them by using the methods in core and in client code
 *
 * 		//Arapi.app.Controller
 *
 * 		this.getArapiModel("User");
 * 
 * 		this.getArapiView("UserWindow");
 * 
 * Arapi.app.Controller also provides inheritance for refs
 * 
 * For example:
 *
 * 		//Arapi.app.Controller
 *
 * 		//Core code
 * 		arapiRefs: [
 * 			{
 * 				ref: "UsersWindow",
 * 				selector: "userwindow",
 * 				autoCreate: true,
 * 				height: 400,
 * 				width: 300,
 * 				modal: true
 * 			},
 * 			{
 * 				ref: "ProductsPanel",
 * 				selector: "viewport > productpanel",
 * 				inheritancePorperty: "this will be overridden in inheritance",
 * 				superProperty: "this should be present in inheritance",
 * 				autoCreate: true
 * 			},
 * 			{
 * 				ref: "CategoryForm",
 * 				selector: "viewport > categorypanel > #catform"
 * 			}
 * 		]
 * 
 * 
 * 		//Client code
 * 		arapiRefs: [
 * 			{
 * 				ref: "ProductsPanel",
 * 				selector: "viewport > clientproductpanel",
 * 				inheritancePorperty: "the client value"
 * 			},
 * 			{
 * 				ref: "CategoryForm",
 * 				selector: "viewport > clientcategorypanel > #clientcatform"
 * 			}
 * 		]
 * 
 * while you can access in both core and client by the getters
 *
 * 		//Arapi.app.Controller
 *		
 * 		this.getUsersWindow();
 * 		this.getProductsPanel();
 * 		this.getCategoryForm();
 * 
 * 
 * Furthermore the configuration properties "inheritancePorperty" and "superProperty" of ProductsPanel arapiRef will still exists in 
 * client extension while "inheritancePorperty" will be overridden with "the client value".
 * 
 * 
 * Arapi.app.Controller provides new or extended features for refs (arapiRefs) to make usage more efficient:
 * 
 * <ul>
 * 	<li><b>selector (extended):</b> In Arapi selector must be a unique component query to avoid getting different components</li>
 * 	<li><b>className (new):</b> In an autoCreate or forceCreate ref you can use className as used in arapiViews instead of using xtype. If className is defined you don't need to define selector property to access the instance. Adventage: Class will be loaded automatically by Ext.Loader</li>
 * 	<li><b>targetSelector (new):</b> In an autoCreate or forceCreate ref you can define an UNIQUE component selector where the new created component will be added (using @link Ext.container.AbstractContainer#add) automatically</li>
 * 	<li><b>nSelector (new):</b> As selector must be unique in Arapi you can use nSelector to get an array of resulting components. Will not work with forceCreate or autoCreate.</li>
 * </ul>
 * 
 * 
 * Using Arapi.app.Application you must use arapiStores, arapiModels, arapiViews and arapiRefs instead of stores, models, views and refs property.
 * 
 * For examples, docs, download and unit tests have a look at <a href="http://www.alexisdorn.de/arapi-ext">http://www.alexisdorn.de/arapi-ext</a>.
 * 
 * Author: Alexis Dorn
 * 
 */

Ext.define("Arapi.app.Controller", {
	extend: "Ext.app.Controller",
	/**
	 * Will be called in class definition pipeline as preprocessor.
	 * Enables inheritance of arapiRefs. 
	 * Enables inheritance of array definition of refs, views, stores and models
	 * Enables defining models, stores and views being object with className and ref
	 * Avoids reference of refs (as in native ext), store, models and views 
	 * @param {Function} cls Subclass
	 * @param {Object} data Data passed into Ext.define
	 * @private
	 */
	onClassExtended: function(cls, data, hooks) {
		var extendFn = function (cls, data){
			var superCls = cls.superclass,
				mixedCollection,
				testCollection,
				testMixedObject,
				addItem,
				i = 0,
				j = 0,
				ln,
				onBeforeClassCreated = (Ext.isEmpty(hooks)) ? data.onBeforeClassCreated : hooks.onBeforeCreated,
				inheritanceItems = ["refs", "models", "stores", "views"],
				superProp = {},
				prop = {},
				item,
				arapiItem,
				beforeCreateFn
			;
			
			//######################################
			//Merge arapiRefs with parent arapiRefs
			for (i = 0; i < inheritanceItems.length; i++){
				arapiItem = "arapi" + Ext.String.capitalize(inheritanceItems[i]);
				item = inheritanceItems[i];
				superProp[arapiItem] = superCls[arapiItem] || [];
				prop[arapiItem] = data[arapiItem] || [];
				
				prop[arapiItem] = superProp[arapiItem].concat(prop[arapiItem] );
				
				mixedCollection = new Ext.util.MixedCollection(false, function(mItem){
					return Ext.String.capitalize(mItem.ref);
				});
				testCollection = {};
				
				for (j = 0, ln = prop[arapiItem].length; j < ln; ++j) {
					testMixedObject = testCollection[prop[arapiItem][j].ref];
					if (!Ext.isEmpty(testMixedObject)){
						addItem = Ext.apply(testMixedObject, prop[arapiItem][j]);
					}else{
						addItem = prop[arapiItem][j];
					}
					mixedCollection.add(Ext.clone(addItem));
					
					testCollection[prop[arapiItem][j].ref] = prop[arapiItem][j];
	            }
				data[arapiItem] = mixedCollection.getRange();
				
				//Setting Date for views, models and stores
				if (arapiItem != "arapiRefs"){
					prop[item] = data[item] || [];
					for (j = 0, ln = data[arapiItem].length; j < ln; ++j) {
						prop[item].push(data[arapiItem][j]["className"]);
		            }
				}
				
			}
			
			//######################################
			//Merge refs, models, stores and views with parent setting
			for (i = 0; i < inheritanceItems.length; i++){
				item = inheritanceItems[i];
				superProp[item] = superCls[item] || [];
				prop[item] = data[item] || [];
				data[item] = Ext.Array.unique(superProp[item].concat(prop[item]));
			}
			
			
			beforeCreateFn = function(cls, data) {
				var module,
					i = 0,
					j = 0,
					item,
					items,
					prefix,
					requires = [],
					modules = [
						{
							prop: "arapiRefs",
							target : "view"
						},
						{
							prop: "arapiViews",
							target : "view"
						},
						{
							prop: "arapiModels",
							target : "model"
						},
						{
							prop: "arapiStores",
							target : "store"
						}
					],
					className = Ext.getClassName(cls),
					namespace = Ext.Loader.getPrefix(className),
					arapiCls = "";
				;
				
				for (i = 0; i < modules.length; i++){
					module = modules[i];
					items = Ext.Array.from(data[module.prop]);
					for (j = 0; j < items.length; j++){
						item = items[j];
						//prefix = Ext.Loader.getPrefix(item);
						if (!Ext.isEmpty(item["className"])){
							arapiCls = namespace + "." + module.target + "." + item["className"];
							requires.push(arapiCls);
							if (module.prop == "arapiRefs"){
								item["arapiViewCls"] = arapiCls;
							}
						}	
					}
				}
				Ext.require(requires, Ext.Function.pass(onBeforeClassCreated, arguments, this));
			};
			if (Ext.isEmpty(hooks)){
				data.onBeforeClassCreated = beforeCreateFn;
			}else{
				hooks.onBeforeCreated = beforeCreateFn;
			}
			
			
		};
		
		//Setting onClassExtended
		data.onClassExtended = function (){
			extendFn.apply(this || window, arguments);
			//return cls.superclass.superclass.$onExtended.apply(this || window, arguments);
		};
		extendFn.apply(this || window, arguments);
	},
	/**
	 * @private
	 * @param cfg
	 */
	constructor: function (cfg){
		this.callParent(arguments);
		if (this.arapiRefs) {
            this.arapiRef(this.arapiRefs);
        }
	},
	/**
	 * Returns same value as this.getView but by ref defined in arapiViews property
	 * @param ref
	 * @return {Function}
	 */
	getArapiView: function (ref){
		return this.getArapiItemsHelper("view", ref);
	},
	/**
	 * Returns same value as this.getStore but by ref defined in arapiStores property
	 * @param ref
	 * @return {Function}
	 */
	getArapiStore: function (ref){
		return this.getArapiItemsHelper("store", ref);
	},
	/**
	 * Returns same value as this.getModel but by ref defined in arapiModels property
	 * @param ref
	 * @return {Function}
	 */
	getArapiModel: function (ref){
		return this.getArapiItemsHelper("model", ref);
	},
	/**
	 * @private
	 * @param getArapiItemsHelper
	 */
	getArapiItemsHelper: function (type, ref){
		var i = 0, 
			items, 
			item, 
			bType = Ext.String.capitalize(type), 
			aType = "arapi" + bType, 
			defProp = aType + "s",
			me = this
		;
		if (Ext.isEmpty(me[aType + "Cache"])){
			me[aType + "Cache"] = {};
		}
		if (Ext.isEmpty(me[aType + "Cache"][ref])){
			items = Ext.Array.from(me[defProp]);
			for (i = 0; i < items.length; i++){
				item = items[i];
				me[aType + "Cache"][item.ref] = item.className;
			}
		}
		if (!Ext.isEmpty(me[aType + "Cache"][ref])){
			return me["get" + bType](me[aType + "Cache"][ref]);
		}
		return me["get" + bType]("");
	},
	/**
	 * @private
	 * @param arapiRefs
	 */
	arapiRef: function (arapiRefs){
		var me = this;
		arapiRefs = Ext.Array.from(arapiRefs);
		Ext.Array.each(arapiRefs, function(info) {
			var ref = info.ref,
				fn = 'get' + Ext.String.capitalize(ref)
			;
			if (!me[fn]){
				me[fn] = Ext.Function.pass(me.arapiGetRef, [ref, info], me);
			}
		});
	},
	/**
	 * Can be defined by className, xtype or selector
	 * className + xtype: autoCreate or forceCreate must be true
	 * className + xtype: targetSelector can be defined to add ref automatically as item. Must be exactly one existing component of selector. Must be a selector for exactly one component
	 * selector: Must be maximal 1 components of selector. Must be a selector for maximal one component. Component must be already instantiated
	 * nSelector: Will return an result of Ext.ComponentQuery.query(nSelector) 
	 * @param ref
	 * @param info
	 * @private
	 */
	arapiGetRef: function (ref, info, cfg){
		this.arapiRefCache = this.arapiRefCache || {};
		var cached = this.arapiRefCache[ref],
			clsName,
			cmp,
			info = info || {},
			cfg = cfg || {}
		;
		info = Ext.apply(info, cfg);
		//The component always has to be instantiated
		if (info.forceCreate){
			//info.className has been defined
			if (Ext.isString(info.arapiViewCls)){
				clsName = info.arapiViewCls;
				cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
			}
			//info.xtype has been defined
			if (Ext.isString(info.xtype)){
				clsName = Ext.ClassManager.getNameByAlias("widget." + info.xtype);
				cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
			}
			//info.targetSelector has been defined
			if (Ext.isString(info.targetSelector)){
				this.arapiAddRef2Target(cached, info.targetSelector);
			}
			return cached;
		}
		if (!cached) {
			if (info.autoCreate){
				//info.className has been defined
				if (!cached && info.autoCreate && Ext.isString(info.arapiViewCls)){
					clsName = info.arapiViewCls;
					this.arapiRefCache[ref] = cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
				}
				//info.xtype has been defined
				if (!cached && info.autoCreate && Ext.isString(info.xtype)){
					clsName = Ext.ClassManager.getNameByAlias("widget." + info.xtype);
					this.arapiRefCache[ref] = cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
				}
				//info.targetSelector has been defined
				if (cached && Ext.isString(info.targetSelector)){
					this.arapiAddRef2Target(cached, info.targetSelector);
				}
			}
			//info.selector has been defined
			if (!cached && Ext.isString(info.selector)){
				cmp = Ext.ComponentQuery.query(info.selector);
				if (cmp.length == 1){
					this.arapiRefCache[ref] = cached = cmp[0];
				}else{
					//TODO: This should be an ArapiError
					Ext.Error.raise({
	                    msg: "More than one or none result found for '" + info.selector + "'"
	                });
				}
			}
			//adding destroy handler to delete caching
			if (cached) {
                cached.on('beforedestroy', function() {
                	this.arapiRefCache.refCache[ref] = null;
                });
            }
			//info.nSelector has been defined
			//there will be no caching of results
			if (!cached && Ext.isString(info.nSelector)){
				cached = Ext.ComponentQuery.query(info.nSelector);
			}
		}
		return cached;
	},
	/**
	 * @param info
	 * @returns {Object}
	 * @private
	 */
	arapiMakeRefConfig: function (info){
		var ownProps = ["arapiViewCls", "autoCreate", "ref", "className", "forceCreate", "xtype", "selector", "targetSelector", "nSelector"],
			cfg = {}
		;
		Ext.Object.each(info, function (key, value){
			if (Ext.Array.indexOf(ownProps, key) == -1){
				cfg[key] = value;
			}
		}, this);
		return cfg;
	},
	/**
	 * 
	 * @param cmp
	 * @param targetSelector
	 * @private
	 */
	arapiAddRef2Target: function (cmp, targetSelector){
		var targetCmp = Ext.ComponentQuery.query(targetSelector);
		if (targetCmp.length == 1){
			targetCmp = targetCmp[0];
			targetCmp.add(cmp);
		}else{
			//TODO: This should be an ArapiError
			Ext.Error.raise({
                msg: "More than one result found for target '" + targetSelector + "'"
            });
		}
	},
	/**
     * @private
     */
	getModel: function (){
		return this.callParent(arguments);
	},
	/**
     * @private
     */
	getStore: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getView: function (){
		return this.callParent(arguments);
	}
	/**
     * Inheritable views of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller views definition</li>
     * </ul>  
     * @property arapiViews
     * @type Objects[]
     */
	/**
     * Inheritable models of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller models definition</li>
     * </ul>  
     * @property arapiModels
     * @type Objects[]
     */
	/**
     * Inheritable stores of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller stores definition</li>
     * </ul>
     * @property arapiStores
     * @type Objects[]
     */
	/**
     * Inheritable refs of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>selector (extended):</b> In Arapi selector must be a unique component query to avoid getting different components</li>
     * 	<li><b>className (new):</b> In an autoCreate or forceCreate ref you can use className as used in arapiViews instead of using xtype. If className is defined you don't need to define selector property to access the instance. . Adventage: Class will be loaded automatically by Ext.Loader</li>
     * 	<li><b>targetSelector (new):</b> In an autoCreate or forceCreate ref you can define an UNIQUE component selector where the new created component will be added (using @link Ext.container.AbstractContainer#add) automatically</li>
     *	<li><b>nSelector (new):</b> As selector must be unique in Arapi you can use nSelector to get an array of resulting components. Will not work with forceCreate or autoCreate.</li>
     *</ul>
     * All other properties will be used as configuration in constructor function if autoCreate or forceCreate is true.  
     * @property arapiRefs
     * @type Objects[]
     */
	/**
     * @cfg models
     * @private
     * @hide 
     */
	/**
     * @cfg stores
     * @private
     * @hide
     */
	/**
     * @cfg views 
     * @private
     * @hide
     */
	/**
     * @cfg refs
     * @private
     * @hide
     */
	
});