/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.JasminePanel
 * @extends Ext.panel.Panel
 * @author Alexis Dorn
 * Tree Panel with Arapi.unittest.plugin.JasmineSuite plugin. 
 * Output of jasmine test results in a tree view.
 * Allows defining the scope of test functions. 
 * With Arapi.unittest.JasminePanel you can run unit tests everywhere inside your application. 
 * 
 * Include jasmine.js from <a href="http://pivotal.github.com/jasmine" target="_blank">http://pivotal.github.com/jasmine</a>
 * 
 * Easy to use API:
 * 
 *     // Be sure that jasmine.js (http://pivotal.github.com/jasmine) 
 *     // is included in your application
 *     var uTestPanel = Ext.create("Arapi.unittest.JasminePanel", {
 *         jasmineCfg: {
 *             fn: function (){
 *                 var me = this;
 *                 describe("Suite 1", function(){
 *                     it("that's ok", function(){
 *                         expect(1 == 1).toBeTruthy();
 *                     });
 *                     it("that's not ok", function(){
 *                         expect(1 != 1).toBeTruthy();
 *                     });
 *                 });
 *             },
 *             scope: this
 *         }
 *     });
 * 		
 *     // myContainer.add(uTestPanel);
 * 		
 *     // Run test with output into tree of jasmine panel
 *     uTestPanel.run();
 * 		
 * 
 * For a tutorial of jasmine have a look at <a href="http://evanhahn.com/?p=181" target="_blank">How do I Jasmine</a> from Evan Hahn.
 * 
 * Tested with jasmine-1.1.0
 */
Ext.define("Arapi.unittest.JasminePanel", {
	requires: [
		"Arapi.unittest.reporter.JasmineTreeReporter",
		"Arapi.unittest.plugin.JasmineSuite",
		"Ext.tip.*"
	],
	xtype: "jasmineunittestpanel",
	extend: "Ext.tree.Panel",
	/**
	 * 
	 * @param cfg
	 */
	constructor: function (cfg){
		var me = this;
		
		if (Ext.isEmpty(cfg.reporterClass)){
			/**
			 * @cfg {String} reporterClass Defaults to 'Arapi.unittest.reporter.JasmineTreeReporter'
			 */
			cfg.reporterClass = "Arapi.unittest.reporter.JasmineTreeReporter";
		}
		
		/**
		 * @cfg {Object} jasmineCfg 
		 * Same configuration used with {@link Arapi.unittest.plugin.JasmineSuite}. For example:
		 * 		jasmineCfg: {
		 *       	title: "Title",
		 *     		fn: function (){describe("Suite 1", {(...)})},
		 * 			scope: this
		 * 		}
		 */
		
		//creating reporter
		this.reporter = Ext.create(cfg.reporterClass, {});
		
		cfg = Ext.apply({}, cfg, {
			jasmineCfg: {},
			//Panel cfg
			rootVisible: true,
			store: this.reporter.getTreeStore(),
			columns: [
				{
					text: "Description",
					dataIndex: "description",
					xtype: 'treecolumn',
					flex: 4
				},
				{
					text: "Passed",
					dataIndex: "passed",
					flex: 1,
					renderer: this.renderPassed
				},
				{
					text: "Error",
					dataIndex: "errorMsg",
					hidden: true
				},
				{
					text: "Stack trace",
					dataIndex: "stackTrace",
					hidden: true
				}
			],
			tbar: [
				{
					text: "Expand All",
					handler: this.expandAll,
					scope: this
				}
			]
		});
		
		//adding suite plugin
		this.addSuitePlugin(cfg.jasmineCfg);
				
		this.callParent([cfg]);
		
		this.addListener ("itemclick", function(view, rec){
	});
		
		this.getView().on('render', function(view) {
		    view.tip = Ext.create('Ext.tip.ToolTip', {
		        target: view.el,
		        delegate: view.itemSelector,
		        trackMouse: true,
		        renderTo: Ext.getBody(),
		        listeners: {
		            beforeshow: function updateTipBody(tip) {
		            	var rec = view.getRecord(tip.triggerElement);
		            	//TODO: could be an suite with errors!!!!
		            	if (Ext.isEmpty(rec.get('errorMsg'))){
		            		tip.update('Results for "' + rec.get('description') + '" => OK');
		            	}else{
		            		tip.update('<b>Errors:</b><br>' + rec.get('errorMsg') + '<br><br><b>Stack trace:</b><br>' + rec.get('stackTrace'));
		            	}
	            		
		            }
		        }
		    });
		});
		
		
	},
	/**
	 * @private
	 */
	addSuitePlugin: function(sCfg){
		this.plugins = this.plugins || [];
		this.plugins.push(Ext.apply({}, sCfg,{
			ptype: "jasminetestsuite",
			reporter: this.reporter
		}));
	},
	/**
	 * Runs jasmine test suite defined in jasmineCfg.fn 
	 */
	run: function (){
		var plugs = this.plugins, i = 0, plug;
		this.reporter.getTreeStore().reset();
		
		for (i = 0; i < plugs.length; i++){
			plug = plugs[i];
			if (plug instanceof Arapi.unittest.plugin.AbstractSuite){
				plug.run(this);
			}
		}
	},
	/**
	 * @private
	 * @param val
	 */
	renderPassed: function (val, obj){
		obj.tdCls = (val == true) ? 'arapi_icon_ok' : 'arapi_icon_error';
		return "";
	}
});