/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.reporter.JasmineTreeReporter
 * @extends Arapi.unittest.reporter.JasmineAbstract
 * @author Alexis Dorn
 * 
 * Output of Jasmine test results in a Tree Store. Implementation of jasmine.Reporter
 */
Ext.define("Arapi.unittest.reporter.JasmineTreeReporter", {
	extend: "Arapi.unittest.reporter.JasmineAbstract",
	requires: [
		"Arapi.unittest.reporter.JasmineTreeStore"
	],
	constructor: function (cfg){
		cfg = Ext.apply({}, cfg, {
			/**
			 * @cfg {String} storeClass Tree store to save results, default to 'Arapi.unittest.reporter.JasmineTreeStore' 
			 */
			storeClass: "Arapi.unittest.reporter.JasmineTreeStore"
		});
		
		this.store = Ext.create(cfg.storeClass, {});
		this.model = this.store.model;
		this.modelName = this.model.$className;
		
		this.callParent([cfg]);
	},
	/**
	 * Returns the tree store
	 * @return {Arapi.unittest.reporter.JasmineTreeStore}
	 */
	getTreeStore: function (){
		return this.store;
	},
	/**
	 * @param {jasmine.Runner} runner
	 * @private
	 */
	reportRunnerStarting: function (runner){
				
	},
	/**
	 * @param {jasmine.Spec} spec
	 * @private
	 */
	reportSpecStarting: function (spec){
		this.addSuite2Tree(spec.suite);
	},
	/**
	 * @private
	 * @param {jasmine.Suite} suite
	 */
	addSuite2Tree: function (suite){
		var pNode = this.store.getRootNode(), 
			sNode, 
			pId = pNode.getId()
		;
		if (this.store.getNodeById(this.suiteIdent + suite.id)){
			return;
		}
		if (suite.parentSuite) {
			pId = this.suiteIdent + suite.parentSuite.id;
			pNode = this.store.getNodeById(pId);
			if (Ext.isEmpty(pNode)){
				this.addSuite2Tree(suite.parentSuite);
			}
			pNode = this.store.getNodeById(pId);
		}
		sNode = Ext.create(this.modelName, {
			id: this.suiteIdent + suite.id,
			pId: pId, 
			description: suite.description,
			type: "suite",
			passed: false
		});
		pNode.appendChild(sNode);
	},
	/**
	 * @param {jasmine.Runner} runner
	 * @private
	 */
	reportRunnerResults: function (runner){
		var results = runner.results(),
			specs = runner.specs(),
			specCount = 0,
			i = 0,
			message = [],
			sNode = this.store.getRootNode(),
			nText = ""
		;
		for (i = 0; i < specs.length; i++) {
			specCount++;
		}
		message = [
			specCount,
			" spec",
			specCount == 1 ? "" : "s",
			", ",		
			results.failedCount,
			" failure",
			(results.failedCount == 1) ? "" : "s"
		].join("");
		
		nText = "Results: " + message;
		
		sNode.set("passed", results.failedCount == 0);
		sNode.set("description", nText);
		sNode.commit();
	},
	/**
	 * @private
	 */
	reportResult2RootNode: function (specs, errors){
		var message, root = this.store.getRootNode(), rootSpecs;
		root.set("nSpecs", root.get("nSpecs") + specs);
		
		if (errors > 0){
			root.set("passed", false);
		}
		rootSpecs = root.get("nSpecs");
		message = [
			"Results for ",
			rootSpecs,
			" spec",
			rootSpecs == 1 ? "" : "s",
		].join("");
		
		root.set("description", message);
		root.commit();
		
		if (!root.isExpanded() && root.hasChildNodes()){
			root.expand();
		}
	},
	/**
	 * @param {jasmine.Suite} suite
	 * @private
	 */
	reportSuiteResults: function (suite){
		var results = suite.results(),
			sNode = this.store.getNodeById(this.suiteIdent + suite.id),
			specs = suite.specs()
		;
		if (Ext.isEmpty(sNode)){
			return;
		}
		sNode.set("passed", results.passed());
		sNode.set("nSpecs", specs.length);
		sNode.set("nErrors", results.failedCount);
		sNode.commit();
		this.reportResult2RootNode(specs.length, results.failedCount);
	},
	/**
	 * @param {jasmine.Spec} spec
	 * @private
	 */
	reportSpecResults: function (spec){
		var results = spec.results(),
			pNode = this.store.getNodeById(this.suiteIdent + spec.suite.id),
			sNode,
			resultItems,
			result,
			i,
			errorMsg = [],
			stackTrace = []
		; 
		
	    
		resultItems = results.getItems();
		for (i = 0; i < resultItems.length; i++) {
			result = resultItems[i];
			if (result.type == 'expect' && result.passed && !result.passed()) {
				errorMsg.push(result.message);
				if (result.trace.stack){
					stackTrace.push(result.trace.stack);
				}
			}
		}
		
		//TODO: Put this in  this.reportSpecStarting and update only passed 
		sNode = Ext.create(this.modelName, {
			id: this.suiteIdent + spec.suite.id + "-spec-" + spec.id,
			pId: pNode.getId(), 
			description: spec.description,
			type: "spec",
			passed: results.passed(),
			leaf: true,
			errorMsg: errorMsg.join(", "),
			stackTrace: stackTrace.join(", ")
		});
		
		pNode.appendChild(sNode);
	},
	/**
	 * @param {String} msg
	 * @private
	 */
	log: function (msg){
		
	},
	/**
	 * tree store
	 * @private
	 */
	store: false,
	/**
	 * root node
	 * @private
	 */
	root: false,
	/**
	 * model class
	 * @private
	 */
	model: false,
	/**
	 * model class name
	 * @private
	 */
	modelName: false,
	/**
	 * @private
	 */
	suiteIdent: "suite-"
});
